;; G. Jones, November 1998.
;; -----------------------
;; A group of general variables that will be used by various different
;; files. 
;; NOTE: For any changes to this file to take effect it must be
;; recompiled, and then general-functions.lisp must be recompiled.
;;
;; CHANGE RECORD
;; Ini  Date      Change summary & functions added, deleted, or altered.
;; ---  --------  ------------------------------------------------------
;; GJ   06/11/98  Initial program.
;;                ------------------------------------------------------
;; GJ   18/11/98  Changed *default-directory* to be a defvar rather than
;;                a defconstant so that it can be reset depending which
;;                project people are working on. Also renamed it to
;;                *project-directory* as this is more meaningful.
;;                ------------------------------------------------------
;; GJ   08/02/99  Changed references to HD to *hard-drive*.
;;                ------------------------------------------------------
;; GJ   03/03/99  Removed defvar for *hard-drive* and *project-directory*
;;                because these will have to have been defined outside of
;;                this file in order for it to be used as part of the
;;                filepath for loading in this file.
;;                ------------------------------------------------------
;; XX   NN/NN/NN  xxx
;; ======== END OF CHANGE RECORD ========


;;---------------------- CONSTANTS -----------------------------;;

;; Character constants.
;; --------------------
(defconstant *new-quote-symbol* "@"
  "Lisp has difficulty with quotes, so they get changed 
   to this symbol.")
(defconstant *open-parentheses* "(")
(defconstant *close-parentheses* ")")
(defconstant *quote* "'")
(defconstant *full-stop* ".")
(defconstant *whitespace* " ")
(defconstant *ascii-char-zero* 48)
(defconstant *ascii-char-nine* 57)
(defconstant *ascii-char-little-a* 97)
(defconstant *ascii-char-little-z* 122)
(defconstant *ascii-char-big-a* 65)
(defconstant *ascii-char-big-z* 90)
(defconstant *ascii-char-space* 32)
(defconstant *word-separator* ""
  "Character which represents the break between each sequence of
   phonemes that represent a word. For example, THE CAT would be
   coded in phonemes as DH AH0 @ K AE1 T assuming the separator was @.")

;; File constants.
;; ---------------
(defconstant *phoneme-file-suffix* ".phon"
  "The suffix for the file containing the phonemic representation of
   sentences. This will be suffixed to the input file name.")
(defconstant *non-lexicon-word-file-suffix* ".non-lex-wd"
  "The suffix for the file containing words which are not in the lexicon.
   This will be suffixed to the input file name.")
(defconstant *non-lexicon-sentence-file-suffix* ".non-lex-sn"
  "The suffix for the file containing sentences having words which are
   not in the lexicon. This will be suffixed to the input file name.")
(defconstant *phoneme-frequency-file-suffix* ".pf"
  "The suffix for the file containing the frequency of phonemes in
   a file of phonemic sentences. This will be suffixed to the
   input file name.")
(defconstant *bigram-frequency-file-suffix* ".bf"
  "The suffix for the file containing the frequency of bigrams in
   a file of phonemic sentences. This will be suffixed to the
   input file name.")


;;---------------------- VARIABLES -----------------------------;;

;(defvar *project-directory* (concatenate 'string *hard-drive*
;                                         ":Syntax project:syntax:")
;  "Project directory and hence the default working directory. Will
;   change depending on the project, so should be set in the main
;   file of the project.")
(defvar *directory* nil "Used for current working directory. Usually this
                         will be the same as *project-directory*.")
(defvar *maternal-files-directory* (concatenate 'string
                                                *project-directory*
                                                "Maternal-files:")
  "Directory for maternal files.")
(defvar *child-files-directory* (concatenate 'string
                                              *project-directory*
                                              "Child-files:")
  "Directory for child files.")
(defvar *code-files-directory* (concatenate 'string
                                             *project-directory*
                                             "Code:")
  "Directory for code files.")
(defvar *input-file* nil "Variable for any input file.")
(defvar *output-file* nil "Variable for any output file.")