;; Gary Jones, November 1998.
;; --------------------------
;; Count the number of times each phoneme occurs in
;; a list of phoneme sentences.
;;
;; CHANGE RECORD
;; Ini  Date      Change summary & functions added, deleted, or altered.
;; ---  --------  ------------------------------------------------------
;; GJ   06/11/98  Initial program.
;;                ------------------------------------------------------
;; GJ   18/11/98  Changed referneces for *default-directory* to
;;                *project-directory* as there will be more than one project.
;;                ------------------------------------------------------
;; GJ   01/03/99  Added a check of lisp-implementation-version to determine
;;                whether to load fasl files or pfsl (vsn 4+) files.
;;                ------------------------------------------------------
;; GJ   03/03/99  All lexicon related files will be loaded from
;;                lexicon-loader.lisp now, so the loads for logical-dir and 
;;                general-functions can be commented out. Also filepaths should
;;                use *project-directory*.
;;                ------------------------------------------------------
;; GJ   11/03/99  Before writing to output files, delete the file if it
;;                already exists. This is because overwrite in Lisp
;;                leaves the tail end of the previous file if the
;;                overwritten information is shorter.
;;                ------------------------------------------------------
;; XX   NN/NN/NN  xxx
;; ======== END OF CHANGE RECORD ========


;(cond
;  ((member "4" (mapcar #'string (coerce (lisp-implementation-version) 'list))
;           :test #'equal)
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.pfsl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.pfsl")))
;  (t
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.fasl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.fasl"))))


;;---------------------- VARIABLES -----------------------------;;



;;---------------------- FUNCTIONS -----------------------------;;

(defun count-phonemes ()
  "Count the number of times each phoneme appears in a file of
   phonemic sentences."
  (let ((temp nil) (sentence nil)
        (phoneme-list nil) (count-list-ph nil) (phoneme-count-list nil)
        (bigram-list nil) (count-list-bi nil) (bigram-count-list nil)
        (bigram-file nil))
    (setf phoneme-list '())
    (setf count-list-ph '())
    (setf bigram-list '())
    (setf count-list-bi '())
    (setf *directory* *project-directory*)
    (format t "~%Enter input directory (default: ~A): " *directory*)
    (setf temp (read-line))
    (if (> (length temp) 0)
      (progn
        (setf *directory* temp)
        (setf temp nil)))
    (format t "~%Enter input file (enter=quit) : ")
    (setf temp (read-line))
    (loop while (> (length temp) 0) do
      (setf *input-file*
              (concatenate 'string *directory*
                                   temp))
      (setf *output-file*
              (concatenate 'string *input-file*
                                   *phoneme-frequency-file-suffix*))
      (delete-if-exists *output-file*)
      (setf bigram-file
              (concatenate 'string *input-file*
                                   *bigram-frequency-file-suffix*))
      (delete-if-exists bigram-file)
      (with-open-file (input *input-file*
                             :direction :input)
        (with-open-file (output *output-file*
                                :direction :output)
          (with-open-file (output-bi bigram-file
                                     :direction :output)
            (do ((a-line (read-line input nil 'eof nil)
                         (read-line input nil 'eof nil)))
              ((eq a-line 'eof) 'nil)
                (setf sentence (coerce-string-into-list (string-upcase a-line)))
                (setf phoneme-count-list (update-phoneme-counts-for-sentence
                                           sentence phoneme-list count-list-ph))
                (setf phoneme-list (first phoneme-count-list))
                (setf count-list-ph (second phoneme-count-list))

                (setf bigram-count-list (update-bigram-counts-for-sentence
                                           sentence bigram-list count-list-bi))
                (setf bigram-list (first bigram-count-list))
                (setf count-list-bi (second bigram-count-list)))

            ;; Print out the phonemes and their frequencies.
            (loop while phoneme-list do
              ;; Phoneme-list is a list of the phonemes.
              ;; Count-list-ph is the phoneme counts - they correspond to
              ;; the position of the phonemes in phoneme-list.
              (princ (car phoneme-list) output)
              (princ " " output)
              (princ (car count-list-ph) output)
              (terpri output)
              (setf phoneme-list (rest phoneme-list))
              (setf count-list-ph (rest count-list-ph)))

            ;; Print out the bigrams and their frequencies.
            (loop while bigram-list do
              ;; Bigram-list is a list of the bigrams.
              ;; Count-list-bi is the bigram counts - they correspond to
              ;; the position of the bigrams in bigram-list.
              (princ (car bigram-list) output-bi)
              (princ " " output-bi)
              (princ (car count-list-bi) output-bi)
              (terpri output-bi)
              (setf bigram-list (rest bigram-list))
              (setf count-list-bi (rest count-list-bi))))))
      
      (format t "~%Enter input file (enter=quit) : ")
      (setf temp (read-line)))))


(defun update-phoneme-counts-for-sentence (sentence phoneme-list count-list)
  "Update the phoneme counts for the phonemes in the sentence.
   The phoneme list corresponds to the count-list, so that the
   the position in the phoneme list corresponds to the same
   position in the count list."
  (let ((phoneme nil) (position 0))
    (loop while sentence do
      (setf phoneme (car sentence))
      (cond
        ((not (equal (string phoneme) *word-separator*))
          (cond
            ((member phoneme phoneme-list :test #'equal)
              (setf position (- (length phoneme-list)
                                (length (member phoneme phoneme-list :test #'equal))))
              (setf (nth position count-list) (+ (nth position count-list) 1)))
            (t
              (setf phoneme-list (append phoneme-list
                                         (list phoneme)))
              (setf count-list (append count-list
                                       (list 1)))))))
      (setf sentence (rest sentence)))
    (list phoneme-list count-list)))


(defun update-bigram-counts-for-sentence (sentence bigram-list count-list)
  "Update the bigram counts for the bigrams in the sentence.
   The bigram list corresponds to the count-list, so that the
   the position in the bigram list corresponds to the same
   position in the count list."
  (let ((bigram nil) (position 0))
    (loop while (second sentence) do
      (setf bigram (concatenate 'string (string (first sentence))
                                        " "
                                        (string (second sentence))))
      (cond
        ((not (search *word-separator* bigram))
          (cond
            ((member bigram bigram-list :test #'equal)
              (setf position (- (length bigram-list)
                                (length (member bigram bigram-list :test #'equal))))
              (setf (nth position count-list) (+ (nth position count-list) 1)))
            (t
              (setf bigram-list (append bigram-list
                                        (list bigram)))
              (setf count-list (append count-list
                                       (list 1)))))))
      (setf sentence (rest sentence)))
    (list bigram-list count-list)))