;; File description:
;; Functions to place random words from the lexicon hash-table
;; into a new hash table for input to the model.
;; 
;; CHANGE RECORD
;; Ini  Date      Change summary & functions added, deleted, or altered.
;; ---  --------  ------------------------------------------------------
;; GJ   28/01/99  Initial program.
;;                ------------------------------------------------------
;; GJ   01/03/99  Added a check of lisp-implementation-version to determine
;;                whether to load fasl files or pfsl (vsn 4+) files.
;;                ------------------------------------------------------
;; GJ   03/03/99  All lexicon related files will be loaded from
;;                lexicon-loader.lisp now, so the loads for logical-dir and 
;;                general-functions can be commented out.
;;                ------------------------------------------------------
;; XX   NN/NN/NN  xxx
;; ======== END OF CHANGE RECORD ========


;(cond
;  ((member "4" (mapcar #'string (coerce (lisp-implementation-version) 'list))
;           :test #'equal)
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.pfsl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.pfsl")))
;  (t
;    (load (concatenate 'string  *hard-drive*
;                       ":logical-dir-compatibility.fasl"))
;    (load (concatenate 'string  *project-directory*
;                                "general-functions.fasl"))))
;(def-logical-directory "home;" *project-directory*)


(defvar *hash-random-line* 0)
(defvar *random-number-limit* 5)
(defvar *hash-count* 0)
(defvar *new-hash-count* 0)
(defvar *random-lexicon* nil "Hash table for the random lexicon words.")

;;---------------------- FUNCTIONS -----------------------------;;

(defun init-random-lexicon ()
  "Create the hash-table. Note a rehash-threshold of 1 indicates the table 
   must be completely full before it is expanded."
  (setf *random-lexicon* (make-hash-table :size 80000 :test #'equal
                                          :rehash-threshold 1)))

(defun create-random-lexicon (arg1 arg2)
  ;; Only add to the hash if its appropriate.
  (incf *hash-count*)
;(format t "~%hcount ~A" *hash-count*)
;(format t "~%newcount ~A" *new-hash-count*)
;(format t "~%rndline ~A" *hash-random-line*)
  (cond
    ((>= (truncate *hash-count*) (truncate *hash-random-line*))
      (cond
        ((listp arg1) ;; The first argument isn't always the word!!
          (setf (gethash (truncate *new-hash-count*) *random-lexicon*) (list arg1 arg2)))
        (t
          (setf (gethash (truncate *new-hash-count*) *random-lexicon*) (list arg2 arg1))))
      (setf *hash-random-line* (+ *hash-random-line*
                                  (random *random-number-limit*)))
      (incf *new-hash-count*))))


(defun build-new-hash ()
  (init-random-lexicon)
  (setf *hash-count* 0)
  (setf *new-hash-count* 0)
  (setf *hash-random-line* (random *random-number-limit*))
  (maphash 'create-random-lexicon *hash-table*))